import {Md5} from "ts-md5";
import Cookies from 'js-cookie';
import {Disc} from "radix-icons-svelte";

function getIdent(): string {
    let cookie = Cookies.get("subsonicPlayerIdent");
    if (typeof cookie === "undefined") {
        cookie = Math.random().toString(36).slice(2);
    }
    const options = {
        expires: 7,
        path: "/",
        sameSite: "strict",
    };
    Cookies.set("subsonicPlayerIdent", cookie, options);
    return cookie;
}

export module OpenSubsonic {
    export let username: string = "";
    export let password: string = "";
    let token: string = "";
    let salt: string = "";
    export let base = "https://music.neshweb.net";
    const apiVer = "1.16.1"; // Version supported by Navidrome. Variable for easier updating
    export const clientName = "Lydstyrke-" + getIdent();

    export async function get(path: string, parameters: Array<Parameter> = []) {
        const apiPath = getApiUrl(path, parameters);
        const res = (await fetch(apiPath));
        if (res.ok) {
            const contentType =res.headers.get("content-type");
            switch (contentType) {
                case("application/json"): {
                    return (await res.json())["subsonic-response"];
                }
                case("audio/mp4"): {
                    return res;
                }
                case("image/jpeg"): {
                    return res;
                }
                case("image/png"): {
                    return res;
                }
                default: {

                    console.warn(`Content Type: '${contentType}' is not supported`)
                }
            }
        }
        else {
            return false;
        }
    }

    export function getApiUrl(path: string, parameters: Array<Parameter> = []) {
        let apiPath = generateBasePath(path);
        parameters.forEach((parameter) => {
            apiPath = apiPath + `&${parameter.key}=${parameter.value}`;
        })

        return apiPath;
    }

    const generateBasePath = (path: string) => {
        if (username === "") {
            const cookie = Cookies.get("subsonicUsername");
            if (typeof cookie !== "undefined") {
                username = cookie;
            }
        }

        let cookie = Cookies.get("subsonicToken");
        if (typeof cookie !== "undefined") {
            token = cookie;

            cookie = Cookies.get("subsonicSalt");
            if (typeof cookie !== "undefined") {
                salt = cookie;
            }
        }
        else {
            generateToken();
        }

        return `${base}/rest/${path}?u=${username}&s=${salt}&t=${token}&v=${apiVer}&c=${clientName}&f=json`;
    }

    const generateToken = () => {
        salt = "staticfornow";
        token = Md5.hashStr(password + salt);
    }

    export function storeCredentials() {
        const options = {
            expires: 7,
            path: "/",
            sameSite: "strict",
        };
        Cookies.set("subsonicToken", token, options);
        Cookies.set("subsonicSalt", salt, options);
        Cookies.set("subsonicUsername", username, options);
    }
}

export interface Parameter {
    key: string,
    value: string
}

interface OpenSubsonicResponse {
    status: string,
    version: string,
    type: string,
    serverVersion: string,
    openSubsonic: boolean,
}

export interface Song {
    id: string,
    parent?: string,
    isDir: boolean,
    title?: string,
    album?: string,
    artist?: string,
    track?: number,
    year?: number,
    genre?: string,
    coverArt?: string,
    size?: number,
    contentType?: string,
    suffix?: string,
    transcodedContentType?: string,
    transcodedSuffix?: string,
    duration?: number,
    bitRate?: number,
    bitDepth?: number,
    samplingRate?: number,
    channelCount?: number,
    bitRate?: number,
    path?: string,
    isVideo?: boolean,
    userRating?: number,
    averageRating?: number,
    playCount?: number,
    discNumber?: number,
    created?: string,
    starred?: string,
    albumId?: string,
    artistId?: string,
    type?: string,
    mediaType?: string,
    bookmarkPosition?: number,
    originalWidth?: number,
    originalHeight?: number,
    played?: string,
    bpm?: number,
    comment?: string,
    sortName?: string,
    musicBrainzId?: string,
    genres?: Array<ItemGenre>,
    artists?: Array<ArtistID3>,
    displayArtist?: string,
    albumArtists?: Array<ArtistID3>,
    displayAlbumArtist?: string,
    contributors?: Array<Contributor>
    displayComposer?: string,
    moods?: Array<string>,
    replayGain?: ReplayGain,
}

export interface ItemGenre {
    name: string
}

export interface ReplayGain {
    trackGain?: number,
    albumGain?: number,
    trackPeak?: number,
    albumPeak?: number,
    baseGain?: number,
    fallbackGain?: number,
}

export interface NowPlayingResponse extends OpenSubsonicResponse {
    nowPlaying: NowPlaying
}

interface NowPlaying {
    entry: Array<NowPlayingEntry>
}

export interface NowPlayingEntry extends Song {
    username: string,
    minutesAgo?: string,
    playerId?: number,
    playerName?: string,
}

export interface GetPlayQueueResponse extends OpenSubsonicResponse {
    playQueue: PlayQueue
}

interface PlayQueue extends NowPlaying {
    dummy: string
}

export interface GetAlbumList2Response extends OpenSubsonicResponse {
    albumList2: AlbumList2
}

export interface AlbumList2 {
    album: Array<AlbumID3>
}

export interface AlbumID3 {
    id: string,
    name: string,
    artist?: string,
    artistId?: string,
    coverArt?: string,
    songCount: number,
    duration: number,
    playCount?: number,
    created: string,
    starred?: string,
    year?: number,
    genre?: string,
    played?: string,
    userRating?: number,
    recordLabels?: Array<RecordLabel>,
    musicBrainzId?: string,
    genres?: Array<ItemGenre>,
    artists?: Array<ArtistID3>,
    displayArtist?: string,
    releaseTypes?: Array<string>,
    moods?: Array<string>,
    sortName?: string,
    originalReleaseDate?: ItemDate,
    releaseDate?: ItemDate,
    isCompilation?: boolean,
    discTitles?: Array<DiscTitle>
}

export interface GetAlbumInfo2Response extends OpenSubsonicResponse {
    albumInfo: AlbumInfo
}

export interface AlbumInfo {
    notes?: string,
    musicBrainzId?: string,
    lastFmUrl?: string,
    smallImageUrl?: string,
    mediumImageUrl?: string,
    largeImageUrl?: string,
}

export interface GetAlbumResponse extends OpenSubsonicResponse{
    album: AlbumID3WithSongs
}

export interface AlbumID3WithSongs extends AlbumID3 {
    song?: Array<Song>,
}

interface RecordLabel {
    name: string
}

interface ArtistID3 {
    id: string,
    name: string,
    coverArt?: string,
    artistImageUrl?: string,
    albumCount?: number,
    starred?: string,
    musicBrainzId?: string,
    sortName?: string,
    roles?: Array<string>
}

interface Contributor {
    role: string,
    subRole?: string,
    artist: ArtistID3,
}

interface ItemDate {
    year?: number,
    month?: number,
    day?: number,
}

interface DiscTitle {
    disc: number,
    title: string,
}