import {PlaybackMode, PlaybackStateSvelte} from "$lib/player.svelte";
import {OpenSubsonic, type Parameter, type Song} from "$lib/opensubsonic";
import {queueState} from "$lib/states/play-queue.svelte";

export const shuffle = (array: Array<Song>) => {
    for (let i: number = array.length - 1; i > 0; i--) {
        const j: number = Math.floor(Math.random() * (i + 1));
        [array[i], array[j]] = [array[j], array[i]];
    }
    return array;
};

interface PlaybackState {
    loopMode: PlaybackStateSvelte,
    shuffle: boolean,
    song: HTMLAudioElement,
    metaData: Song,
    duration: number,
    volume: number,
    paused: boolean,
    progress: number,
    newSong: (song: Song) => void,
    play: () => void,
    pause: () => void,
}

export const playbackState: PlaybackState = $state({
    loopMode: new PlaybackStateSvelte(),
    shuffle: false,
    song: {},
    metaData: {},
    duration: 0,
    volume: 0.05,
    paused: true,
    progress: 0,
    newSong(song: Song): void {
        const parameters: Array<Parameter> = [
            { key: "id", value: song.id },
            //{ key: "maxBitRate", value: } // TODO
            //{ key: "format", value: } // TODO
            //{ key: "timeOffset", value: } // TODO? Only Video related
            //{ key: "size", value: } // TODO? Only Video related
            { key: "estimateContentLength", value: "true" },
            //{ key: "converted", value: } // TODO? Only Video related
        ];
        const url = OpenSubsonic.getApiUrl("stream", parameters);
        this.song = new Audio(url); // Assign new URL
        this.metaData = song;

        // Reassign Event Handlers
        this.song.onloadedmetadata = () => {
            this.duration = this.song.duration;
        };

        this.song.onplay = () => {
            this.song.volume = this.volume;
            this.paused = this.song.paused;
            const time: number = Date.now();
            const parameters: Array<Parameter> = [
                { key: "id", value: song.id },
                { key: "time", value: time.toString()},
                { key: "submission", value: `${false}`}
            ];
            OpenSubsonic.get("scrobble", parameters);
        }

        this.song.onpause = () => {
            this.paused = this.song.paused;
        }

        this.song.ontimeupdate = () => {
            this.progress = this.song.currentTime;
        }

        this.song.onended = () => {
            switch (this.loopMode.current) {
                case PlaybackMode.Linear: {
                    this.newSong(queueState.nextSong());
                    this.play();
                    break;
                }
                case PlaybackMode.LoopOne: {
                    this.play();
                    break;
                }
                case PlaybackMode.LoopQueue: {
                    if (queueState.currentIndex === queueState.queue.length -1) {
                        if (this.shuffle) {
                            const shuffledQueue = shuffle([...queueState.queue])
                            queueState.replaceQueue(shuffledQueue);
                        }
                        this.newSong(queueState.firstSong());
                    }
                    else {
                        this.newSong(queueState.nextSong());
                    }
                    this.play();
                    break;
                }
            }
        }

        this.song.load();
    },
    play() {
        this.song.play().catch((): void => {});
    },
    pause() {
        console.log(this.song);
        this.song.pause();
    }
})